@@ -10,6 +10,7 @@ import edgedb
 from devtools import debug
 from pydantic import BaseModel, PrivateAttr, Field
 
+from .unset import is_unset
 from .cache import CacheManager
 from .execute import query as execute_query
 from .batch import Batch
@@ -18,6 +19,8 @@ from . import creating_strings
 from .span import safe_span
 
 InsertType = T.TypeVar("InsertType", bound=BaseModel)
+PatchType = T.TypeVar("PatchType", bound=BaseModel)
+
 ThisNodeType = T.TypeVar("ThisNodeType", bound="Node")
 
 BaseModelType = T.TypeVar("BaseModelType", bound=BaseModel)
@@ -152,7 +155,7 @@ def resolver_equal_or_better(
     return True
 
 
-class Node(BaseModel, T.Generic[InsertType]):
+class Node(BaseModel, T.Generic[InsertType, PatchType]):
     id: UUID = Field(..., allow_mutation=False)
 
     _cache: CacheManager = PrivateAttr(default_factory=CacheManager)
@@ -516,6 +519,19 @@ for item in json_array_unpack(raw_data) union ({insert_str}) {given_resolver.to_
             new_node._deleted = True
             models_by_id[new_node.id].hydrate(new_node=new_node)
 
+    @classmethod
+    def build_update_d_from_patch(cls, patch: PatchType) -> dict:
+        update_d = {}
+        for field in patch.__fields__.keys():
+            val = getattr(patch, field)
+            if not is_unset(val):
+                if val is None:
+                    val = set()
+                elif isinstance(val, BaseModel):
+                    val = val.json()
+                update_d[field] = val
+        return update_d
+
     def build_update_d(self) -> dict:
         fields_to_update = self.__updated_fields__ & self.GraphORM.updatable_fields
         update_d = {}
@@ -570,6 +586,71 @@ for item in json_array_unpack(raw_data) union ({insert_str}) {given_resolver.to_
                     update_d[field_name] = set()
         return update_d
 
+    @classmethod
+    async def _update(
+        cls: T.Type[ThisNodeType],
+        resolver: "ResolverType",
+        update_d: dict,
+        set_links_d: T.Dict[str, "Resolver"] = None,
+        batch: Batch = None,
+        given_client: edgedb.AsyncIOClient = None,
+    ) -> T.Optional[ThisNodeType]:
+        s, variables = model_dict_to_str(
+            d=update_d,
+            edgedb_conversion_map=cls._edgedb_conversion_map,
+            set_links_d=set_links_d,
+        )
+
+        resolver.validate_against_query_variables(variables)
+        update_inner_str = f"UPDATE {cls._model_name} {resolver._filter_str} SET {s}"
+
+        variables_to_use = {**resolver._get_nested_query_variables(), **variables}
+
+        if batch:
+            batch.add(line=update_inner_str, variables=variables_to_use)
+            return None
+
+        update_s = (
+            f"WITH model := ({update_inner_str}) "
+            f"SELECT model {resolver.to_str(include_filters=False)}"
+        )
+
+        with safe_span(op=f"edgedb.update.{cls._model_name}"):
+            raw_d = await execute_query(
+                client=given_client or cls.db_client,
+                query_str=update_s,
+                variables=variables_to_use,
+                only_one=True,
+            )
+        if not raw_d:
+            raise NodeException(f"No update for {cls._model_name} was registered.")
+        with safe_span(op=f"parse.{cls._model_name}"):
+            node = resolver.parse_nested_obj(raw_d)
+        return node
+
+    @classmethod
+    async def update_from_patch(
+        cls: T.Type[ThisNodeType],
+        patch: PatchType,
+        resolver_with_filter: "ResolverType",
+        set_links_d: T.Dict[str, "Resolver"] = None,
+        batch: Batch = None,
+        given_client: edgedb.AsyncIOClient = None,
+    ) -> T.Optional[ThisNodeType]:
+        update_d = cls.build_update_d_from_patch(patch=patch)
+        if not update_d:
+            raise NodeException("No update registered.")
+        node = await cls._update(
+            resolver=resolver_with_filter,
+            update_d=update_d,
+            set_links_d=set_links_d,
+            batch=batch,
+            given_client=given_client,
+        )
+        if batch:
+            return None
+        return node
+
     async def update(
         self,
         given_resolver: "ResolverType" = None,
@@ -592,41 +673,15 @@ for item in json_array_unpack(raw_data) union ({insert_str}) {given_resolver.to_
 
         given_resolver.clear_top_level_filters_and_variables()
         given_resolver.filter(filter_str=".id = <uuid>$id", variables={"id": self.id})
-        s, variables = model_dict_to_str(
-            d=update_d,
-            edgedb_conversion_map=self._edgedb_conversion_map,
+        node = await self._update(
+            resolver=given_resolver,
+            update_d=update_d,
             set_links_d=set_links_d,
+            batch=batch,
+            given_client=given_client,
         )
-
-        given_resolver.validate_against_query_variables(variables)
-        update_inner_str = (
-            f"UPDATE {self._model_name} {given_resolver._filter_str} SET {s}"
-        )
-
-        variables_to_use = {**given_resolver._get_nested_query_variables(), **variables}
-
         if batch:
-            batch.add(line=update_inner_str, variables=variables_to_use)
             return None
-
-        update_s = (
-            f"WITH model := ({update_inner_str}) "
-            f"SELECT model {given_resolver.to_str(include_filters=False)}"
-        )
-
-        with safe_span(op=f"edgedb.update.{self._model_name}"):
-            raw_d = await execute_query(
-                client=given_client or self.db_client,
-                query_str=update_s,
-                variables=variables_to_use,
-                only_one=True,
-            )
-        if not raw_d:
-            raise NodeException(
-                f"No update for {self._model_name}: {self.id} was registered."
-            )
-        with safe_span(op=f"parse.{self._model_name}"):
-            node = given_resolver.parse_nested_obj(raw_d)
         self.hydrate(new_node=node)
 
     @classmethod
